// Copyright 2024 WHERE TRUE Technologies.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

use exon::datasources::sam::table_provider::ListingSAMTableOptions;
use pyo3::{pyclass, pymethods};

#[pyclass]
#[derive(Debug, Clone, Default)]
/// Options for reading BAM files.
pub struct SAMReadOptions {}

#[pymethods]
impl SAMReadOptions {
    #[new]
    pub fn new() -> Self {
        Self::default()
    }
}

impl From<SAMReadOptions> for ListingSAMTableOptions {
    fn from(_options: SAMReadOptions) -> Self {
        ListingSAMTableOptions::default()
    }
}
